package com.zmkm.shop.config.shiro;

import com.zmkm.shop.config.shiro.session.ShiroSession;
import org.apache.shiro.cache.AbstractCacheManager;
import org.apache.shiro.cache.Cache;
import org.apache.shiro.cache.CacheException;
import org.apache.shiro.cache.MapCache;
import org.apache.shiro.session.Session;
import org.apache.shiro.session.UnknownSessionException;
import org.apache.shiro.session.mgt.eis.CachingSessionDAO;

import java.io.Serializable;
import java.util.concurrent.ConcurrentHashMap;

public class CachingShiroSessionDao extends CachingSessionDAO {

	public CachingShiroSessionDao() {
		setCacheManager(new AbstractCacheManager() {
			@Override
			protected Cache<Serializable, Session> createCache(String name) throws CacheException {
				return new MapCache<Serializable, Session>(name, new ConcurrentHashMap<Serializable, Session>());
			}
		});
	}
	
	@Override
	public void update(Session session) throws UnknownSessionException {
		if (session instanceof ShiroSession) {
			// 如果没有主要字段(除lastAccessTime以外其他字段)发生改变
			ShiroSession ss = (ShiroSession) session;
			if (!ss.isChanged()) {
				return;
			}
		}
		super.update(session);
	}

	protected Serializable doCreate(Session session) {
        Serializable sessionId = generateSessionId(session);
        assignSessionId(session, sessionId);
        return sessionId;
    }

    protected Session doReadSession(Serializable sessionId) {
        return null; //should never execute because this implementation relies on parent class to access cache, which
        //is where all sessions reside - it is the cache implementation that determines if the
        //cache is memory only or disk-persistent, etc.
    }

    protected void doUpdate(Session session) {
        //does nothing - parent class persists to cache.
    }

    protected void doDelete(Session session) {
        //does nothing - parent class removes from cache.
    }

}
