package com.zmkm.shop.config.shiro.cache;

import org.apache.shiro.cache.CacheException;
import org.springframework.cache.Cache.ValueWrapper;
import org.springframework.cache.caffeine.CaffeineCacheManager;
import org.springframework.data.redis.core.RedisTemplate;

import java.util.Collection;
import java.util.Collections;
import java.util.Set;
import java.util.concurrent.TimeUnit;

@SuppressWarnings({ "unchecked", "rawtypes"})
public class ShiroSpringCache<K, V> implements org.apache.shiro.cache.Cache<K, V> {

	public static long DEFAULT_TIMEOUT = 1;
	
	private long timeout;
	
	private String cacheName = "";

	private CaffeineCacheManager firstCache;

	private RedisTemplate redisTemplate;

	public ShiroSpringCache(String cacheName, CaffeineCacheManager guavaCacheManager, RedisTemplate redisTemplate) {
		this.timeout = DEFAULT_TIMEOUT;
		this.cacheName = cacheName;
		this.firstCache = guavaCacheManager;
		this.redisTemplate = redisTemplate;
	}
	
	public ShiroSpringCache(String cacheName, CaffeineCacheManager guavaCacheManager, RedisTemplate redisTemplate, long timeout) {
		this.timeout = DEFAULT_TIMEOUT;
		this.cacheName = cacheName;
		this.firstCache = guavaCacheManager;
		this.redisTemplate = redisTemplate;
	}

	@Override
	public V get(K key) throws CacheException {
//		Long start = System.currentTimeMillis();
		if (key == null) {
			return null;
		}
		ValueWrapper valueWrapper = this.firstCache.getCache(cacheName).get(key);
		if (valueWrapper != null) {
//			System.out.println("查询["+key+"]耗时：" + (System.currentTimeMillis() - start));
			return (V) valueWrapper.get();
		}
		Object result = this.redisTemplate.opsForValue().get(cacheName + key);
		if(null != result) {
			this.firstCache.getCache(cacheName).put(key, result);
		}
//		System.out.println("查询["+key+"]耗时：" + (System.currentTimeMillis() - start));
		return (V) result;
	}

	@Override
	public V put(K key, V value) throws CacheException {
//		System.out.println("put:" + key);
		V previous = get(key);
		this.firstCache.getCache(cacheName).put(key, value);
		this.redisTemplate.opsForValue().set(cacheName + key, value);
		this.redisTemplate.expire(cacheName + key, this.timeout, TimeUnit.DAYS);
		return previous;
	}

	@Override
	public V remove(K key) throws CacheException {
		V previous = get(key);
		this.firstCache.getCache(cacheName).evict(key);
		this.redisTemplate.delete(cacheName + key);
		return previous;
	}

	@Override
	public void clear() throws CacheException {
		this.firstCache.getCache(cacheName).clear();
	}

	@Override
	public int size() {
		return 0;
	}

	@Override
	public Set<K> keys() {
		return Collections.emptySet();
	}

	@Override
	public Collection<V> values() {
		return Collections.emptySet();
	}
	
}
