package com.zmkm.shop.config.shiro.cache;

import org.apache.shiro.cache.Cache;
import org.apache.shiro.cache.CacheException;
import org.apache.shiro.cache.CacheManager;
import org.apache.shiro.util.Destroyable;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.cache.caffeine.CaffeineCacheManager;
import org.springframework.data.redis.core.RedisTemplate;

@SuppressWarnings({ "rawtypes" })
public class ShiroSpringCacheManager implements CacheManager, Destroyable {
	private Logger logger = LoggerFactory.getLogger(ShiroSpringCacheManager.class);

	private CaffeineCacheManager caffeineCacheManager;
	private RedisTemplate redisTemplate;
	private long timeout;

	public void setCaffeineCacheManager(CaffeineCacheManager caffeineCacheManager) {
		this.caffeineCacheManager = caffeineCacheManager;
	}

	public void setRedisTemplate(RedisTemplate redisTemplate) {
		this.redisTemplate = redisTemplate;
	}

	public void setTimeout(long timeout) {
		this.timeout = timeout;
	}

	@Override
	public <K, V> Cache<K, V> getCache(String name) throws CacheException {
		if (logger.isTraceEnabled()) {
			logger.trace("Acquiring ShiroSpringCache instance named [" + name + "]");
		}
		if(timeout <= 0) {
			return new ShiroSpringCache<K, V>(name, caffeineCacheManager, redisTemplate);
		}
		return new ShiroSpringCache<K, V>(name, caffeineCacheManager, redisTemplate, timeout);
	}

	@Override
	public void destroy() throws Exception {
	}

}
